@@ -50,6 +50,8 @@
 
 /* Private part of the WnckClassGroup structure */
 struct _WnckClassGroupPrivate {
+  WnckScreen *screen;
+
   char *res_class;
   char *name;
   GList *windows;
@@ -220,6 +222,7 @@ wnck_class_group_get (const char *id)
 
 /**
  * _wnck_class_group_create:
+ * @screen: a #WnckScreen.
  * @res_class: name of the resource class for the group.
  *
  * Creates a new WnckClassGroup with the specified resource class name.  If
@@ -230,7 +233,8 @@ wnck_class_group_get (const char *id)
  * matches the @res_class.
  **/
 WnckClassGroup *
-_wnck_class_group_create (const char *res_class)
+_wnck_class_group_create (WnckScreen *screen,
+                          const char *res_class)
 {
   WnckClassGroup *class_group;
 
@@ -242,6 +246,7 @@ _wnck_class_group_create (const char *res_class)
 			NULL);
 
   class_group = g_object_new (WNCK_TYPE_CLASS_GROUP, NULL);
+  class_group->priv->screen = screen;
 
   class_group->priv->res_class = g_strdup (res_class ? res_class : "");
 
@@ -451,10 +456,8 @@ set_icon (WnckClassGroup *class_group)
   if (!icon || !mini_icon)
     {
       _wnck_get_fallback_icons (&icon,
-                                _wnck_get_default_icon_size (),
                                 _wnck_get_default_icon_size (),
                                 &mini_icon,
-                                _wnck_get_default_mini_icon_size (),
                                 _wnck_get_default_mini_icon_size ());
       icons_reffed = TRUE;
     }
@@ -496,6 +499,20 @@ update_class_group_name (WnckWindow     *window,
   set_name (class_group);
 }
 
+static void
+window_weak_notify_cb (gpointer  data,
+                       GObject  *where_the_window_was)
+{
+  WnckClassGroup *class_group;
+  WnckClassGroupPrivate *priv;
+
+  class_group = WNCK_CLASS_GROUP (data);
+  priv = class_group->priv;
+
+  g_hash_table_remove (priv->window_icon_handlers, where_the_window_was);
+  g_hash_table_remove (priv->window_name_handlers, where_the_window_was);
+}
+
 /**
  * _wnck_class_group_add_window:
  * @class_group: a #WnckClassGroup.
@@ -534,6 +551,8 @@ _wnck_class_group_add_window (WnckClassGroup *class_group,
                        window,
                        (gpointer) signal_id);
 
+  g_object_weak_ref (G_OBJECT (window), window_weak_notify_cb, class_group);
+
   set_name (class_group);
   set_icon (class_group);
 
