@@ -30,6 +30,7 @@
 #include "xutils.h"
 #include "pager.h"
 #include "util.h"
+#include "wnck-handle-private.h"
 #ifdef HAVE_STARTUP_NOTIFICATION
 #include <libsn/sn.h>
 #endif
@@ -38,6 +39,8 @@ G_BEGIN_DECLS
 
 #define WNCK_ACTIVATE_TIMEOUT 1
 
+WnckHandle *_wnck_get_handle (void);
+
 WnckClientType _wnck_get_client_type (void);
 
 gsize _wnck_get_default_icon_size (void);
@@ -95,8 +98,9 @@ void             _wnck_application_destroy (WnckApplication *app);
 void             _wnck_application_load_icons (WnckApplication *app);
 void             _wnck_application_shutdown_all (void);
 
+WnckClassGroup  *_wnck_class_group_create        (WnckScreen     *screen,
+                                                  const char     *res_class);
 
-WnckClassGroup*  _wnck_class_group_create        (const char     *res_class);
 void             _wnck_class_group_destroy       (WnckClassGroup *class_group);
 void             _wnck_class_group_add_window    (WnckClassGroup *class_group,
                                                   WnckWindow     *window);
@@ -144,8 +148,10 @@ void           _wnck_pager_get_workspace_rect   (WnckPager     *pager,
 void           _make_gtk_label_bold   (GtkLabel *label);
 void           _make_gtk_label_normal (GtkLabel *label);
 
-void           _wnck_stock_icons_init (void);
+void           _wnck_selector_set_window_icon   (GtkWidget     *image,
+                                                 WnckWindow    *window);
 
+void           _wnck_ensure_fallback_style      (void);
 
 G_END_DECLS
 
