@@ -16,9 +16,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -26,10 +24,12 @@
 #include <string.h>
 #include <stdio.h>
 #include <cairo-xlib.h>
+#if HAVE_CAIRO_XLIB_XRENDER
+#include <cairo-xlib-xrender.h>
+#endif
 #include "screen.h"
 #include "window.h"
 #include "private.h"
-#include "inlinepixbufs.h"
 
 gboolean
 _wnck_get_cardinal (Screen *screen,
@@ -699,14 +699,24 @@ _wnck_set_utf8_list (Screen  *screen,
 void
 _wnck_error_trap_push (Display *display)
 {
-  gdk_error_trap_push ();
+  GdkDisplay *gdk_display;
+
+  gdk_display = gdk_x11_lookup_xdisplay (display);
+  g_assert (gdk_display != NULL);
+
+  gdk_x11_display_error_trap_push (gdk_display);
 }
 
 int
 _wnck_error_trap_pop (Display *display)
 {
-  gdk_flush ();
-  return gdk_error_trap_pop ();
+  GdkDisplay *gdk_display;
+
+  gdk_display = gdk_x11_lookup_xdisplay (display);
+  g_assert (gdk_display != NULL);
+
+  gdk_display_flush (gdk_display);
+  return gdk_x11_display_error_trap_pop (gdk_display);
 }
 
 static GdkFilterReturn
@@ -784,6 +794,9 @@ filter_func (GdkXEvent  *gdkxevent,
         }
 #endif /* HAVE_STARTUP_NOTIFICATION */
       break;
+
+    default:
+      break;
     }
 
   return GDK_FILTER_CONTINUE;
@@ -826,7 +839,7 @@ _wnck_xid_hash (gconstpointer v)
   gulong val = * (const gulong *) v;
 
   /* I'm not sure this works so well. */
-#if G_SIZEOF_LONG > 4
+#if GLIB_SIZEOF_LONG > 4
   return (guint) (val ^ (val >> 32));
 #else
   return val;
@@ -869,16 +882,18 @@ _wnck_deiconify (Screen *screen,
 }
 
 void
-_wnck_close (Screen *screen,
-	     Window  xwindow,
-	     Time    timestamp)
+_wnck_close (WnckScreen *screen,
+             Window      xwindow,
+             Time        timestamp)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -915,15 +930,17 @@ _wnck_close (Screen *screen,
 #define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10
 
 void
-_wnck_keyboard_move (Screen *screen,
-                     Window  xwindow)
+_wnck_keyboard_move (WnckScreen *screen,
+                     Window      xwindow)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -948,15 +965,17 @@ _wnck_keyboard_move (Screen *screen,
 }
 
 void
-_wnck_keyboard_size (Screen *screen,
-                     Window  xwindow)
+_wnck_keyboard_size (WnckScreen *screen,
+                     Window      xwindow)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -981,22 +1000,24 @@ _wnck_keyboard_size (Screen *screen,
 }
 
 void
-_wnck_change_state (Screen  *screen,
-		    Window   xwindow,
-                    gboolean add,
-                    Atom     state1,
-                    Atom     state2)
+_wnck_change_state (WnckScreen *screen,
+                    Window      xwindow,
+                    gboolean    add,
+                    Atom        state1,
+                    Atom        state2)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
 #define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
 #define _NET_WM_STATE_ADD           1    /* add/set property */
 #define _NET_WM_STATE_TOGGLE        2    /* toggle property  */
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1021,16 +1042,18 @@ _wnck_change_state (Screen  *screen,
 }
 
 void
-_wnck_change_workspace (Screen     *screen,
-			Window      xwindow,
+_wnck_change_workspace (WnckScreen *screen,
+                        Window      xwindow,
                         int         new_space)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1055,20 +1078,22 @@ _wnck_change_workspace (Screen     *screen,
 }
 
 void
-_wnck_activate (Screen *screen,
-                Window  xwindow,
-                Time    timestamp)
+_wnck_activate (WnckScreen *screen,
+                Window      xwindow,
+                Time        timestamp)
 {
+  Screen *xscreen;
   Display *display;
-  Window   root;
-  XEvent   xev;
+  Window root;
+  XEvent xev;
 
   if (timestamp == 0)
     g_warning ("Received a timestamp of 0; window activation may not "
                "function properly.\n");
 
-  display = DisplayOfScreen (screen);
-  root = RootWindowOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
+  root = RootWindowOfScreen (xscreen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1367,6 +1392,23 @@ _wnck_get_frame_extents (Screen *screen,
       retval = TRUE;
     }
 
+  if (p_size == NULL)
+    {
+      _wnck_get_cardinal_list (screen, xwindow,
+                               _wnck_atom_get ("_GTK_FRAME_EXTENTS"),
+                               &p_size, &n_size);
+
+      if (p_size != NULL && n_size == 4)
+        {
+          *left_frame   = -p_size[0];
+          *right_frame  = -p_size[1];
+          *top_frame    = -p_size[2];
+          *bottom_frame = -p_size[3];
+
+          retval = TRUE;
+        }
+    }
+
   if (p_size != NULL)
     g_free (p_size);
 
@@ -1409,46 +1451,10 @@ _wnck_select_input (Screen *screen,
   return old_mask;
 }
 
-/* The icon-reading code is copied
- * from metacity, please sync bugfixes
- */
-static gboolean
-find_largest_sizes (gulong *data,
-                    gulong  nitems,
-                    int    *width,
-                    int    *height)
-{
-  *width = 0;
-  *height = 0;
-
-  while (nitems > 0)
-    {
-      int w, h;
-
-      if (nitems < 3)
-        return FALSE; /* no space for w, h */
-
-      w = data[0];
-      h = data[1];
-
-      if (nitems < ((w * h) + 2))
-        return FALSE; /* not enough data */
-
-      *width = MAX (w, *width);
-      *height = MAX (h, *height);
-
-      data += (w * h) + 2;
-      nitems -= (w * h) + 2;
-    }
-
-  return TRUE;
-}
-
 static gboolean
 find_best_size (gulong  *data,
                 gulong   nitems,
-                int      ideal_width,
-                int      ideal_height,
+                int      ideal_size,
                 int     *width,
                 int     *height,
                 gulong **start)
@@ -1456,20 +1462,11 @@ find_best_size (gulong  *data,
   int best_w;
   int best_h;
   gulong *best_start;
-  int max_width, max_height;
 
   *width = 0;
   *height = 0;
   *start = NULL;
 
-  if (!find_largest_sizes (data, nitems, &max_width, &max_height))
-    return FALSE;
-
-  if (ideal_width < 0)
-    ideal_width = max_width;
-  if (ideal_height < 0)
-    ideal_height = max_height;
-
   best_w = 0;
   best_h = 0;
   best_start = NULL;
@@ -1487,7 +1484,7 @@ find_best_size (gulong  *data,
       w = data[0];
       h = data[1];
 
-      if (nitems < ((w * h) + 2))
+      if (nitems < ((gulong) (w * h) + 2))
         break; /* not enough data */
 
       if (best_start == NULL)
@@ -1497,7 +1494,6 @@ find_best_size (gulong  *data,
       else
         {
           /* work with averages */
-          const int ideal_size = (ideal_width + ideal_height) / 2;
           int best_size = (best_w + best_h) / 2;
           int this_size = (w + h) / 2;
 
@@ -1573,18 +1569,16 @@ argbdata_to_pixdata (gulong *argb_data, int len, guchar **pixdata)
 }
 
 static gboolean
-read_rgb_icon (Screen        *screen,
-               Window         xwindow,
-               int            ideal_width,
-               int            ideal_height,
-               int            ideal_mini_width,
-               int            ideal_mini_height,
-               int           *width,
-               int           *height,
-               guchar       **pixdata,
-               int           *mini_width,
-               int           *mini_height,
-               guchar       **mini_pixdata)
+read_rgb_icon (Screen  *screen,
+               Window   xwindow,
+               int      ideal_size,
+               int      ideal_mini_size,
+               int     *width,
+               int     *height,
+               guchar **pixdata,
+               int     *mini_width,
+               int     *mini_height,
+               guchar **mini_pixdata)
 {
   Display *display;
   Atom type;
@@ -1622,16 +1616,14 @@ read_rgb_icon (Screen        *screen,
       return FALSE;
     }
 
-  if (!find_best_size (data, nitems,
-                       ideal_width, ideal_height,
-                       &w, &h, &best))
+  if (!find_best_size (data, nitems, ideal_size, &w, &h, &best))
     {
       XFree (data);
       return FALSE;
     }
 
   if (!find_best_size (data, nitems,
-                       ideal_mini_width, ideal_mini_height,
+                       ideal_mini_size,
                        &mini_w, &mini_h, &best_mini))
     {
       XFree (data);
@@ -1691,10 +1683,34 @@ _wnck_cairo_surface_get_from_pixmap (Screen *screen,
       if (!XGetWindowAttributes (display, root_return, &attrs))
         goto TRAP_POP;
 
-      surface = cairo_xlib_surface_create (display,
-                                           xpixmap,
-                                           attrs.visual,
-                                           w_ret, h_ret);
+      if (depth_ret == (unsigned int) attrs.depth)
+	{
+	  surface = cairo_xlib_surface_create (display,
+					       xpixmap,
+					       attrs.visual,
+					       w_ret, h_ret);
+	}
+      else
+	{
+#if HAVE_CAIRO_XLIB_XRENDER
+	  int std;
+
+	  switch (depth_ret) {
+	  case 1: std = PictStandardA1; break;
+	  case 4: std = PictStandardA4; break;
+	  case 8: std = PictStandardA8; break;
+	  case 24: std = PictStandardRGB24; break;
+	  case 32: std = PictStandardARGB32; break;
+	  default: goto TRAP_POP;
+	  }
+
+	  surface = cairo_xlib_surface_create_with_xrender_format (display,
+								   xpixmap,
+								   attrs.screen,
+								   XRenderFindStandardFormat (display, std),
+								   w_ret, h_ret);
+#endif
+	}
     }
 
 TRAP_POP:
@@ -1730,13 +1746,12 @@ try_pixmap_and_mask (Screen     *screen,
                      Pixmap      src_pixmap,
                      Pixmap      src_mask,
                      GdkPixbuf **iconp,
-                     int         ideal_width,
-                     int         ideal_height,
+                     int         ideal_size,
                      GdkPixbuf **mini_iconp,
-                     int         ideal_mini_width,
-                     int         ideal_mini_height)
+                     int         ideal_mini_size)
 {
   cairo_surface_t *surface, *mask_surface, *image;
+  GdkDisplay *gdk_display;
   GdkPixbuf *unscaled;
   int width, height;
   cairo_t *cr;
@@ -1754,6 +1769,11 @@ try_pixmap_and_mask (Screen     *screen,
   if (surface == NULL)
     return FALSE;
 
+  gdk_display = gdk_x11_lookup_xdisplay (XDisplayOfScreen (screen));
+  g_assert (gdk_display != NULL);
+
+  gdk_x11_display_error_trap_push (gdk_display);
+
   width = cairo_xlib_surface_get_width (surface);
   height = cairo_xlib_surface_get_height (surface);
 
@@ -1792,6 +1812,12 @@ try_pixmap_and_mask (Screen     *screen,
   cairo_surface_destroy (surface);
   cairo_destroy (cr);
 
+  if (gdk_x11_display_error_trap_pop (gdk_display) != Success)
+    {
+      cairo_surface_destroy (image);
+      return FALSE;
+    }
+
   unscaled = gdk_pixbuf_get_from_surface (image,
                                           0, 0,
                                           width, height);
@@ -1802,17 +1828,13 @@ try_pixmap_and_mask (Screen     *screen,
     {
       *iconp =
         gdk_pixbuf_scale_simple (unscaled,
-                                 ideal_width > 0 ? ideal_width :
-                                 gdk_pixbuf_get_width (unscaled),
-                                 ideal_height > 0 ? ideal_height :
-                                 gdk_pixbuf_get_height (unscaled),
+                                 ideal_size,
+                                 ideal_size,
                                  GDK_INTERP_BILINEAR);
       *mini_iconp =
         gdk_pixbuf_scale_simple (unscaled,
-                                 ideal_mini_width > 0 ? ideal_mini_width :
-                                 gdk_pixbuf_get_width (unscaled),
-                                 ideal_mini_height > 0 ? ideal_mini_height :
-                                 gdk_pixbuf_get_height (unscaled),
+                                 ideal_mini_size,
+                                 ideal_mini_size,
                                  GDK_INTERP_BILINEAR);
 
       g_object_unref (G_OBJECT (unscaled));
@@ -1890,10 +1912,8 @@ struct _WnckIconCache
   Pixmap prev_mask;
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
-  int ideal_width;
-  int ideal_height;
-  int ideal_mini_width;
-  int ideal_mini_height;
+  int ideal_size;
+  int ideal_mini_size;
   guint want_fallback : 1;
   /* TRUE if these props have changed */
   guint wm_hints_dirty : 1;
@@ -1912,10 +1932,8 @@ _wnck_icon_cache_new (void)
   icon_cache->prev_pixmap = None;
   icon_cache->icon = NULL;
   icon_cache->mini_icon = NULL;
-  icon_cache->ideal_width = -1; /* won't be a legit width */
-  icon_cache->ideal_height = -1;
-  icon_cache->ideal_mini_width = -1;
-  icon_cache->ideal_mini_height = -1;
+  icon_cache->ideal_size = -1; /* won't be a legit size */
+  icon_cache->ideal_mini_size = -1;
   icon_cache->want_fallback = TRUE;
   icon_cache->wm_hints_dirty = TRUE;
   icon_cache->kwm_win_icon_dirty = TRUE;
@@ -2081,16 +2099,15 @@ scaled_from_pixdata (guchar *pixdata,
 }
 
 gboolean
-_wnck_read_icons (Screen        *screen,
-                  Window         xwindow,
-                  WnckIconCache *icon_cache,
-                  GdkPixbuf    **iconp,
-                  int            ideal_width,
-                  int            ideal_height,
-                  GdkPixbuf    **mini_iconp,
-                  int            ideal_mini_width,
-                  int            ideal_mini_height)
+_wnck_read_icons (WnckScreen     *screen,
+                  Window          xwindow,
+                  WnckIconCache  *icon_cache,
+                  GdkPixbuf     **iconp,
+                  int             ideal_size,
+                  GdkPixbuf     **mini_iconp,
+                  int             ideal_mini_size)
 {
+  Screen *xscreen;
   Display *display;
   guchar *pixdata;
   int w, h;
@@ -2104,21 +2121,18 @@ _wnck_read_icons (Screen        *screen,
 
   g_return_val_if_fail (icon_cache != NULL, FALSE);
 
-  display = DisplayOfScreen (screen);
+  xscreen = _wnck_screen_get_xscreen (screen);
+  display = DisplayOfScreen (xscreen);
 
   *iconp = NULL;
   *mini_iconp = NULL;
 
-  if (ideal_width != icon_cache->ideal_width ||
-      ideal_height != icon_cache->ideal_height ||
-      ideal_mini_width != icon_cache->ideal_mini_width ||
-      ideal_mini_height != icon_cache->ideal_mini_height)
+  if (ideal_size != icon_cache->ideal_size ||
+      ideal_mini_size != icon_cache->ideal_mini_size)
     clear_icon_cache (icon_cache, TRUE);
 
-  icon_cache->ideal_width = ideal_width;
-  icon_cache->ideal_height = ideal_height;
-  icon_cache->ideal_mini_width = ideal_mini_width;
-  icon_cache->ideal_mini_height = ideal_mini_height;
+  icon_cache->ideal_size = ideal_size;
+  icon_cache->ideal_mini_size = ideal_mini_size;
 
   if (!_wnck_icon_cache_get_icon_invalidated (icon_cache))
     return FALSE; /* we have no new info to use */
@@ -2140,16 +2154,16 @@ _wnck_read_icons (Screen        *screen,
     {
       icon_cache->net_wm_icon_dirty = FALSE;
 
-      if (read_rgb_icon (screen, xwindow,
-                         ideal_width, ideal_height,
-                         ideal_mini_width, ideal_mini_height,
+      if (read_rgb_icon (xscreen, xwindow,
+                         ideal_size,
+                         ideal_mini_size,
                          &w, &h, &pixdata,
                          &mini_w, &mini_h, &mini_pixdata))
         {
-          *iconp = scaled_from_pixdata (pixdata, w, h, ideal_width, ideal_height);
+          *iconp = scaled_from_pixdata (pixdata, w, h, ideal_size, ideal_size);
 
           *mini_iconp = scaled_from_pixdata (mini_pixdata, mini_w, mini_h,
-                                             ideal_mini_width, ideal_mini_height);
+                                             ideal_mini_size, ideal_mini_size);
 
           replace_cache (icon_cache, USING_NET_WM_ICON,
                          *iconp, *mini_iconp);
@@ -2187,9 +2201,9 @@ _wnck_read_icons (Screen        *screen,
            mask != icon_cache->prev_mask) &&
           pixmap != None)
         {
-          if (try_pixmap_and_mask (screen, pixmap, mask,
-                                   iconp, ideal_width, ideal_height,
-                                   mini_iconp, ideal_mini_width, ideal_mini_height))
+          if (try_pixmap_and_mask (xscreen, pixmap, mask,
+                                   iconp, ideal_size,
+                                   mini_iconp, ideal_mini_size))
             {
               icon_cache->prev_pixmap = pixmap;
               icon_cache->prev_mask = mask;
@@ -2207,15 +2221,15 @@ _wnck_read_icons (Screen        *screen,
     {
       icon_cache->kwm_win_icon_dirty = FALSE;
 
-      get_kwm_win_icon (screen, xwindow, &pixmap, &mask);
+      get_kwm_win_icon (xscreen, xwindow, &pixmap, &mask);
 
       if ((pixmap != icon_cache->prev_pixmap ||
            mask != icon_cache->prev_mask) &&
           pixmap != None)
         {
-          if (try_pixmap_and_mask (screen, pixmap, mask,
-                                   iconp, ideal_width, ideal_height,
-                                   mini_iconp, ideal_mini_width, ideal_mini_height))
+          if (try_pixmap_and_mask (xscreen, pixmap, mask,
+                                   iconp, ideal_size,
+                                   mini_iconp, ideal_mini_size))
             {
               icon_cache->prev_pixmap = pixmap;
               icon_cache->prev_mask = mask;
@@ -2232,11 +2246,9 @@ _wnck_read_icons (Screen        *screen,
       icon_cache->origin < USING_FALLBACK_ICON)
     {
       _wnck_get_fallback_icons (iconp,
-                                ideal_width,
-                                ideal_height,
+                                ideal_size,
                                 mini_iconp,
-                                ideal_mini_width,
-                                ideal_mini_height);
+                                ideal_mini_size);
 
       replace_cache (icon_cache, USING_FALLBACK_ICON,
                      *iconp, *mini_iconp);
@@ -2258,21 +2270,16 @@ _wnck_read_icons (Screen        *screen,
 }
 
 static GdkPixbuf*
-default_icon_at_size (int width,
-                      int height)
+default_icon_at_size (int size)
 {
-
   GdkPixbuf *base;
 
-  base = gdk_pixbuf_new_from_inline (-1, default_icon_data,
-                                     FALSE,
-                                     NULL);
+  base = gdk_pixbuf_new_from_resource ("/org/gnome/libwnck/default_icon.png", NULL);
 
   g_assert (base);
 
-  if ((width < 0 && height < 0) ||
-      (gdk_pixbuf_get_width (base) == width &&
-       gdk_pixbuf_get_height (base) == height))
+  if (gdk_pixbuf_get_width (base) == size &&
+      gdk_pixbuf_get_height (base) == size)
     {
       return base;
     }
@@ -2280,13 +2287,7 @@ default_icon_at_size (int width,
     {
       GdkPixbuf *scaled;
 
-      scaled = gdk_pixbuf_scale_simple (base,
-                                        width > 0 ? width :
-                                        gdk_pixbuf_get_width (base),
-                                        height > 0 ? height :
-                                        gdk_pixbuf_get_height (base),
-                                        GDK_INTERP_BILINEAR);
-
+      scaled = gdk_pixbuf_scale_simple (base, size, size, GDK_INTERP_BILINEAR);
       g_object_unref (G_OBJECT (base));
 
       return scaled;
@@ -2295,26 +2296,17 @@ default_icon_at_size (int width,
 
 void
 _wnck_get_fallback_icons (GdkPixbuf **iconp,
-                          int         ideal_width,
-                          int         ideal_height,
+                          int         ideal_size,
                           GdkPixbuf **mini_iconp,
-                          int         ideal_mini_width,
-                          int         ideal_mini_height)
+                          int         ideal_mini_size)
 {
   if (iconp)
-    *iconp = default_icon_at_size (ideal_width > 0 ? ideal_width :
-                                   DEFAULT_ICON_WIDTH,
-                                   ideal_height > 0 ? ideal_height :
-                                   DEFAULT_ICON_HEIGHT);
+    *iconp = default_icon_at_size (ideal_size);
 
   if (mini_iconp)
-    *mini_iconp = default_icon_at_size (ideal_mini_width > 0 ? ideal_mini_width :
-                                        DEFAULT_MINI_ICON_WIDTH,
-                                        ideal_mini_height > 0 ? ideal_mini_height :
-                                        DEFAULT_MINI_ICON_HEIGHT);
+    *mini_iconp = default_icon_at_size (ideal_mini_size);
 }
 
-
 void
 _wnck_get_window_geometry (Screen *screen,
 			   Window  xwindow,
